@@ -22,7 +22,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 
-__version__ = "0.3.38"
+__version__ = "0.3.39"
 __author__ = "nagev"
 __license__ = "GPLv3"
 
@@ -97,7 +97,6 @@ def new(url, basic=True, gdata=False, signature=True, size=False,
     fetched if the videos have encrypted signatures, so will override the
     value set in the signature argument.
 
-
     """
 
     return Pafy(url, basic, gdata, signature, size, callback)
@@ -112,7 +111,7 @@ class g(object):
         'watchv': "http://www.youtube.com/watch?v=%s",
         'vidinfo': ('http://www.youtube.com/get_video_info?'
                     'video_id=%s&asv=3&el=detailpage&hl=en_US'),
-        'playlist': ('http://www.youtube.com/list_ajax?',
+        'playlist': ('http://www.youtube.com/list_ajax?'
                      'style=json&action_get_list=1&list=%s')
     }
     ua = ("Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64;"
@@ -197,7 +196,6 @@ def _extract_function_from_js(name, js):
     m = re.search(fpattern % re.escape(name), js)
     args, body = m.groups()
     dbg("extracted function %s(%s){%s};", name, args, body)
-    time.sleep(1)
     func = {'name': name, 'parameters': args.split(","), 'body': body}
     return func
 
@@ -368,6 +366,7 @@ def extract_video_id(url):
 
     ok = (r"\w-",) * 3
     regx = re.compile(r'(?:^|[^%s]+)([%s]{11})(?:[^%s]+|$)' % ok)
+    url = str(url)
     m = regx.search(url)
 
     if not m:
@@ -388,7 +387,7 @@ def get_video_info(video_id):
 
     if info['status'][0] == "fail":
         reason = info['reason'][0] or "Bad video argument"
-        raise IOError("Youtube says: %s" % reason)
+        raise IOError("Youtube says: %s [%s]" % (reason, video_id))
 
     return info
 
@@ -398,7 +397,7 @@ def get_video_gdata(video_id):
 
     new.callback("Fetching video gdata")
     url = g.urls['gdata'] % video_id
-    gdata = g.opener.open(g.urls['gdata'] % video_id).read()
+    gdata = g.opener.open(url).read()
     dbg("Fetched video gdata")
     new.callback("Fetched video gdata")
     return gdata
@@ -701,6 +700,7 @@ class Pafy(object):
         self._description = None
         self._category = None
         self._published = None
+        self._username = None
 
         self._sm = []
         self._asm = []
@@ -725,6 +725,8 @@ class Pafy(object):
         self._bigthumb = None
         self._viewcount = None
         self._bigthumbhd = None
+        self.expiry = None
+        self.playlist_meta = None
 
         if self._init_args['basic']:
             self._fetch_basic()
@@ -762,13 +764,11 @@ class Pafy(object):
         self._length = int(f('length_seconds'))
         self._viewcount = int(f('view_count'))
         self._thumb = unquote_plus(f('thumbnail_url'))
-        self._duration = time.strftime('%H:%M:%S', time.gmtime(self._length))
         self._formats = [x.split("/") for x in f('fmt_list').split(",")]
         self._keywords = z('keywords').split(',')
         self._bigthumb = z('iurlsd')
         self._bigthumbhd = z('iurlsdmaxres')
         self.ciphertag = f("use_cipher_signature") == "True"
-        self.expiry = None
 
         if self.ciphertag:
             dbg("Encrypted signature detected.")
@@ -795,6 +795,8 @@ class Pafy(object):
         published = tree.find(t1 + "published").text
         description = groups.find(t0 + "description").text
         category = groups.find(t0 + "category").text
+        username = tree.find(t1 + "author/" + t1 + "uri").text.split("/")[-1]
+        setattr(self, "_username", username)
         setattr(self, "_published", published)
         setattr(self, "_description", description)
         setattr(self, "_category", category)
@@ -884,35 +886,45 @@ class Pafy(object):
     def title(self):
         """ Return YouTube video title as a string. """
 
-        self._fetch_basic()
+        if not self._title:
+            self._fetch_basic()
+
         return self._title
 
     @property
     def author(self):
         """ The uploader of the video. Returns str. """
 
-        self._fetch_basic()
+        if not self._author:
+            self._fetch_basic()
+
         return self._author
 
     @property
     def rating(self):
         """ Rating for a video. Returns float. """
 
-        self._fetch_basic()
+        if not self._rating:
+            self._fetch_basic()
+
         return self._rating
 
     @property
     def length(self):
         """ Length of a video in seconds. Returns int. """
 
-        self._fetch_basic()
+        if not self._length:
+            self._fetch_basic()
+
         return self._length
 
     @property
     def viewcount(self):
         """ Number of views for a video. Returns int. """
 
-        self._fetch_basic()
+        if not self._viewcount:
+            self._fetch_basic()
+
         return self._viewcount
 
     @property
@@ -933,14 +945,19 @@ class Pafy(object):
     def thumb(self):
         """ Thumbnail image url. Returns str. """
 
-        self._fetch_basic()
+        if not self._thumb:
+            self._fetch_basic()
+
         return self._thumb
 
     @property
     def duration(self):
         """ Duration of a video (HH:MM:SS). Returns str. """
 
-        self._fetch_basic()
+        if not self._length:
+            self._fetch_basic()
+
+        self._duration = time.strftime('%H:%M:%S', time.gmtime(self._length))
         return self._duration
 
     @property
@@ -961,9 +978,18 @@ class Pafy(object):
     def description(self):
         """ Description of the video. Returns string. """
 
-        self._fetch_gdata()
+        if not self._description:
+            self._fetch_gdata()
+
         return self._description
 
+    @property
+    def username(self):
+        """ Return the username of the uploader. """
+
+        self._fetch_gdata()
+        return self._username
+
     @property
     def published(self):
         """ The upload date and time of the video. Returns string. """
@@ -1023,22 +1049,42 @@ class Pafy(object):
         else:
             return r
 
+    def populate_from_playlist(self, pl_data):
+        """ Populate Pafy object with items fetched from playlist data. """
 
-def get_playlist(playlist_url, callback=None):
-    """ Return a list of Pafy objects from a YouTube Playlist. """
+        self._title = pl_data.get("title")
+        self._author = pl_data.get("author")
+        self._length = int(pl_data.get("length_seconds", 0))
+        self._rating = pl_data.get("rating", 0.0)
+        self._viewcount = int(pl_data.get("views", 0).replace(",", ""))
+        self._thumb = pl_data.get("thumbnail")
+        self._description = pl_data.get("description")
+        self.playlist_meta = pl_data
 
+
+def get_playlist(playlist_url, basic=False, gdata=False, signature=False,
+                 size=False, callback=None):
+    """ Return a dict containing Pafy objects from a YouTube Playlist.
+
+    The returned Pafy objects are initialised using the arguments to
+    get_playlist() in the manner documented for pafy.new()
+
+    """
+
+    # pylint: disable=R0914
+    # too many local vars
     nullf = lambda x: None
     callback = callback or nullf
-    ok = (r"\w-",) * 3
-    regx = re.compile(r'(?:^|[^%s]+)([%s]{18})(?:[^%s]+|$)' % ok)
+    x = (r"-_0-9a-zA-Z",) * 2 + (r'(?:\&|\#.{1,1000})',)
+    regx = re.compile(r'(?:^|[^%s]+)([%s]{18,})(?:%s|$)' % x)
     m = regx.search(playlist_url)
 
     if not m:
-        err = "Need 18 character video id or the URL of the video. Got %s"
+        err = "Unrecognized playlist url: %s"
         raise ValueError(err % playlist_url)
 
-    playlistid = m.groups(0)
-    url = g.urls["playlist"] % playlistid
+    playlist_id = m.groups(0)
+    url = g.urls["playlist"] % playlist_id
 
     try:
         allinfo = json.loads(decode_if_py3(g.opener.open(url).read()))
@@ -1046,18 +1092,57 @@ def get_playlist(playlist_url, callback=None):
     except:
         raise IOError("Error fetching playlist %s" % m.groups(0))
 
-    videos = []
+    playlist = dict(
+        playlist_id=playlist_id,
+        likes=allinfo.get('likes'),
+        title=allinfo.get('title'),
+        author=allinfo.get('author'),
+        dislikes=allinfo.get('dislikes'),
+        description=allinfo.get('description'),
+        items=[]
+    )
 
     for v in allinfo['video']:
 
+        vid_data = dict(
+            added=v.get('added'),
+            is_cc=v.get('is_cc'),
+            is_hd=v.get('is_hd'),
+            likes=v.get('likes'),
+            title=v.get('title'),
+            views=v.get('views'),
+            rating=v.get('rating'),
+            author=v.get('author'),
+            user_id=v.get('user_id'),
+            privacy=v.get('privacy'),
+            dislikes=v.get('dislikes'),
+            duration=v.get('duration'),
+            comments=v.get('comments'),
+            keywords=v.get('keywords'),
+            thumbnail=v.get('thumbnail'),
+            cc_license=v.get('cc_license'),
+            category_id=v.get('category_id'),
+            description=v.get('description'),
+            encrypted_id=v.get('encrypted_id'),
+            time_created=v.get('time_created'),
+            length_seconds=v.get('length_seconds')
+        )
+
         try:
-            video = Pafy(v['encrypted_id'])
-            callback("Added video: %s" % v.title)
-            videos.append(video)
+            pafy_obj = new(vid_data['encrypted_id'],
+                           basic=basic,
+                           gdata=gdata,
+                           signature=signature,
+                           size=size,
+                           callback=callback)
 
         except IOError as e:
-
             callback("%s: %s" % (v['title'], e.message))
             continue
 
-    return videos
+        pafy_obj.populate_from_playlist(vid_data)
+        playlist['items'].append(dict(pafy=pafy_obj,
+                                      playlist_meta=vid_data))
+        callback("Added video: %s" % v['title'])
+
+    return playlist
